#ifndef CANVAS_H
#define CANVAS_H

#include <QWidget>
#include <QPaintEvent>
#include <QResizeEvent>
#include <QPainterPath>
#include <QPainter>
#include <QString>
#include <QVector>
#include <QTextOption>
#include <QRect>
#include <QTimer>
#include <cmath>
#include "tournament.h"


/**
* @class Canvas
*
* @brief  Draws the Tournament tree, Team names, icons, and Picks
*/ 
class Canvas : public QWidget
{
	Q_OBJECT

public:
	/**
	* \param parent is the parent widget of Canvas
	*/
	Canvas(QWidget *parent);
	~Canvas();
	
	/// animation step
	static int STEP_UP;
	static int STEP_DOWN;
	
	/**
	* The Tournament which should be drawn
	*/
	Tournament *tournament;
	/** 
	* The position of the slider 
	*/
	int round;
	/// picks visible
	bool picks;
	
	/** 
	* Top left corner of the Bracket P(x0,y0), width = dx and heigth = dy 
	*/ 
	int x0, y0, dx, dy;

	/**
	* Animation variables
	*/
	QTimer* timer;
	Game* animatedGame;
	int step;
	bool animated;
	void setAnimationEndPoint(int end, int offset, int color);
	
	/**
	* @brief Sets up the canvas for the first time.
	* 
	* Initialises all important coordinates (x0,y0,dx,dy).
	*/
	void init(void);

protected:
	void mousePressEvent(QMouseEvent* mevent);

private:
	/// Left side
	static const int LEFT = 1;
	/// Right side
	static const int RIGHT = -1;
	/// Vertical margin of the Canvas
	static const int VMARGIN = 50;
	/// Horizontal margin (space between Bracket and Team names)
	static const int brcktHmargin = 10;
	static const int pickHeight = 12;
	static const int normalLen = 20;

	/// Horizontal margin of the Canvas
	int HMARGIN;
	/// Vertical margin between two brackets
	int brcktVmargin;
	
	// animation variables
	/// Animation endpoint
	int animEnd;
	int dashOffset;
	/// Color interpolation step
	int colorStep;

	QPainter *painter;
	QPen pen;
	/// Pen for undecided games
	QPen undecided;
	///Pen for decided games
	QPen decided;
	int fontheight;
	
	/**
	* @brief Draws the Canvas.
	* Always called when something has changed in the data.
	*/
	void paintEvent(QPaintEvent* pevent);
	void resizeEvent(QResizeEvent* revent);
	/**
	* Recursively initializes Bracket positions
	* \param id ID of the brackets
	* \param num Number brackets per level
	* \param y0 Top left corner of the Bracket
	* \param dy Height of the Bracket
	* \param level Depth level in the Tournament tree
	*/
	void initBracketPositions(int id, int num, int y0, int dy, int level);
	QPainterPath drawBracket(int x0, int y0, int ylink, int dx, int dy, int dashOffset);
	void drawBrackets(void);
	/// draws Team names and icons
	void drawTeamInfo (void);
	/// draw Picks
	void drawPicks(int round);
	/**
	* Updates parent brackets during animation
	* \param ID ID of parent bracket
	*/
	void updateParentBracket(int ID);
	/**
	* Indicates if an decided Game is already drawn
	*/
	bool isDrawn(Game* game);
	QPen getPenForTeam(Team* team);
	Team* findTeamFromCoordinates(int x, int y);


private slots:
		void animate();
};

#endif // CANVAS_H
