#ifndef GAME_H
#define GAME_H

#include "team.h"
#include <QPen>

/**
* @class Game
*
* @brief Contains the data of one Game
*
* This class contains all informations that is needed
* for a game, as the two opponents, the result, the date, etc.
* It also contains some informations about the position in the canvas
* (which game is the left/right one, which coordinates does the game
* have) and informations who has won, etc.
*/ 
class Game
{

public:
	/**
	* Creates an empty Game.
	*/
	Game(void);

	/**
	* Creates a Game with Teams and Result.
	*/
	Game(int ID, Team* team1, Team* team2, int result[2]);

	/**
	* Creates a Game with Teams, Result and children.
	*/
	Game(int ID, Team* team1, Team* team2, int result[2], Game* leftChild, Game* rightChild);

	/**
	* Creates a Game with Teams, Result, date and description.
	*/
	Game(int ID, Team* team1, Team* team2, int result[2], QString date, QString descr);
	~Game(void);
	
	/// Game is undecided
	static const int UNDECIDED = 0;
	/// Team 1 wins
	static const int WINTEAM1 = 1;
	/// Team 2 wins
	static const int WINTEAM2 = 2;
	/// Left child game
	static const int LEFTCHILD = 0;
	/// Right child game
	static const int RIGHTCHILD = 1;
	/// Game has no child games
	static const int UNDEFINED = 0;
		
	/**
	* Contains the Result of a Game.
	*/
	struct Result {
		int score[2];
	};

	/**
	* Contains informations about the position of the Game in the Canvas (graphical representation of the game).
	*/
	struct Bracket {
		/// Top left corner
		int y0;
		/// Height
		int dy;
		/// Link point to Parent Bracket
		int ylink;
		/// Depth level in Tournament Tree
		int level;
		/// Left or right Bracket
		int dir;
		/// Dash line offset
		int dashOffset;
		QPen pen;
	};
	
	// getters
	struct Bracket getBracket(void) const;
	int getID (void) const;
	int getState(void) const;
	QString getDate(void);
	QString getDescr(void);
	/**
	* Returns the Team with teamnum 1 or 2.
	*/
	Team* getTeam (int teamnum) const;
	struct Result getResult(void) const;
	/**
	* Returns a child of this Game.
	* dir = 0: leftchild
	* dir = 1: rightchild
	*/
	Game* getChild(int dir) const;
	/**
	* Returns if the Games of the children are decided yet.
	*/
	bool isChildGamesDecided(void);

	/**
	* Returns the ID of Team 1.
	*/
	int getTeam1(void);

	/**
	* Returns the ID of Team 2.
	*/
	int getTeam2(void);

	/**
	* Returns Team 1.
	*/
	Team* getTeam1Direct(void);

	/**
	* Returns Team 2.
	*/
	Team* getTeam2Direct(void);

	//setters
	/**
	* Sets the data for the Bracket.
	*/
	void setBracket(int y0, int dy, int ylink, int level, int dir, int dashOffset, QPen pen);

	/**
	* Sets the data for the Bracket.
	*/
	void setBracket(int y0, int dy, int ylink, int dashOffset);	

	/**
	* Sets the Team with teamnum 1 or 2 (first Team or second Team)
	*/
	void setTeam(int teamnum, Team* team);
	void setTeams(Team* team1, Team* team2); 
	void setResult(int score[2]);
	void setChilds(Game* leftChild, Game* rightChild); 
	void setPen(QPen pen);
	void setPenColor(int val);

private:
	int ID;
	Team* team1;
	Team* team2;
	struct Result result;
	struct Bracket bracket;
	Game* leftChild;
	Game* rightChild;
	QString date;
	QString descr;
	
};

#endif
